# Voice Acting Cutting Tool

This TCoAaL-mod logs text event meta data, which can be used to cut recordings and to create voice acting mods.

## Usage

### Log Event Data while playing the game

As you play the game with this mod enabled, The following data will be logged for reach text event:

- `linesKey` - The key referring to the lines entry in the dialogue.loc.
- `labelText` - The label text of the text Box (e.g. Andrew or Ashley).
- `linesText` - The text shown in the text box.
- `timeStart` - The start time of the event relative to start time of the game. It's a string formatted as HH:MM:SS.sss.
- `timeDuration` - The duration the text box is shown is also formatted as a HH:MM:SS.sss time string.

### Cut Recordings

The logged data can be used to cut recordings into audio files, which cover the time frame the text box has been displayed.
To do this, you need to copy the video and/or audio recordings you want to process into the `/tomb/mods/voiceactingcuttingtool_data/process_sessions` folder. You'll also have to open the `processAudio.config` in a text editor of your choice (preferably one that support JSON syntax highlighting - e.g. [VS Code](https://code.visualstudio.com/)) and modify it to your needs.
```
{
  "sessions": [
    {
      "name": "session1",
      "recordings": [
        {
          "fileName": "aDifferentRecordingFromTheSameSession.mp4",
          "timeOffset": "00:00:00.000",
          "options": "-b:a 128k",
          "labelsToProcess": [
            "Narrator",
            "Ashley"
          ]
        }
      ]
    }
  ],
  "log": [
    "FILEWRITTEN"
  ]
}
```
`sessions` - is an array of logs storing information needed to cut the recordings. Each session has the following properties:
- `name` - the name of the session logs stored in the `/tomb/mods/voiceactingcuttingtool_data/sessions` folder.
- `recordings` - an array of recordings which have been taken during a session. It has got the following properties:
	- `fileName` - the name of the recording file
	- `timeOffset` - describes the difference between the start of the session and the start of the audio file. As a point of reference: A session starts about the time the main menu appears or when the main menu is refreshed after pressing the `Cut Rec` - button. If the recording is started after the sessions, the timestamps logged in the session<number>.json files may be useful to synchronize the audio with the gameplay.
	- `options` - optional options to the ffmpeg command used to cut the audio files. Can be set to `""` or removed if you don't want to use them.
	- `labelsToProcess` - if you don't want to cut the entire recording, but just one or a few characters, you can specify them in this array. Unless it's the Narrator, the labels need to be identical to what's displayed in the top of text box. Any `?` needs to be replaced with `_QUESTIONMARK`. If this property is removed audio files will be created for all text boxes.
- `log` - sets what kind of messages you'd like to see recorded in the processAudio.log file. Currently that's either `FILEWRITTEN` and/or `DEBUG`.

Once the files are in place and configured, press the `Cut Rec` button in the main menu of the game!

### Create a Voice Acting Mod

After you've processed the recordings, you can assemble the cuts into a mod which can be used together with [Basil's Voice Acting Framework](https://codeberg.org/basil/coffin-mods/releases) to add your voice acting to the game.

Open the modCreation.config in a text editor of your choice (again preferably one with JSON Syntax highlighting).
```
{
  "dataForModJson": {
    "id": "my_va_mod",
    "name": "My 1st VA mod",
    "authors": [
      "Your Name"
    ],
    "description": "Enter a description here!",
    "version": "1.0.0",
    "dependencies": {
      "game": "*",
      "spec": "0.1.0",
      "voiceacting": ">=0.4.0"
    }
  },
  "modSubFolderForVoiceActingFiles": "voiceacting/english",
  "audioFileSources": [
    {
      "folder": "output/session1/replaceThisWithYourRecording"
    },
    {
      "folder": "output/session1/aDifferentRecordingFromTheSameSession",
      "subFolders": [
        "Narrator",
        "Ashley"
      ]
    },
    {
      "folder": "manualReplacements",
      "subFolders": [
        "Ashley"
      ]
    }
  ]
}
```
`dataForModJson` - contains data which will be used to create the `mod.json` for your mod. Refer to the [Tomb README.md](https://codeberg.org/basil/tomb) for details.
`modSubFolderForVoiceActingFiles` - the folder in the mod where the audio files will be stored. Take a look at the [Voice Acting Framework README.md](https://codeberg.org/basil/coffin-mods/src/branch/main/voiceactingframework) for details.
`audioFileSources` - an array of audio sources, you'd like to use for your mod. Duplicates will be overridden in order.

After everything is configured, press the `Cuts2Mod` Button in the main menu to create your mod in the `/tomb/mods/` folder. To play it, you'll have to restart your game.

## Installation

You can [download the latest release here](https://codeberg.org/canopy/voiceactingcuttingtool/releases). To use this mod an installation of [Tomb](https://codeberg.org/basil/tomb) is required. Installation instructions for Tomb can be found in the [modding wiki](https://coffin-wiki.basil.cafe/playing/install-a-modloader#prebuilt). With Tomb installed, you can copy the voiceactingcuttingtool folder into the `/tomb/mods` folder. This will cause the mod to be automatically load during the next start of the game. The file structure should look similar to this, where the voiceactingcuttingtool folder can (but doesn't have to) be a zip file:
```
The Coffin of Andy and Leyley/
├── tomb/
│   ├── index.html
│   ├── mods/
│   │   ├── voiceactingcuttingtool/
│   │   │	└── js/
│   │   │	└── LICENSE
│   │   │	└── mod.json
│   │   │	└── README.md
│   │   └── tomb/
│   └── tomb/
```

### Additional requirements

To use the cutting feature of the mod [ffmpeg](https://www.ffmpeg.org/download.html) needs to be installed. Odds are this condition is already met, if you use any audio/video editing software.

## Disclaimer

Generative AI has been used to write code for this mod.